"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const body_parser_1 = __importDefault(require("body-parser"));
const express_async_handler_1 = __importDefault(require("express-async-handler"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const hasha_1 = __importDefault(require("hasha"));
const path_1 = __importDefault(require("path"));
async function applyMiddlewares(app, options) {
    const prefix = options.prefix || "";
    app.get(`${prefix}/api/v4/projects/:projectId/repository/tree`, tree(options));
    app.get(`${prefix}/api/v4/projects/:projectId/repository/files/*`, readFile(options));
    app.post(`${prefix}/api/v4/projects/:projectId/repository/commits`, body_parser_1.default.json(), commit(options));
}
exports.applyMiddlewares = applyMiddlewares;
async function fileToTree(filePath, options) {
    const absFilePath = path_1.default.join(options.cwd, filePath);
    if (!(await fs_extra_1.default.pathExists(absFilePath))) {
        return;
    }
    const stat = await fs_extra_1.default.stat(absFilePath);
    if (stat.isDirectory()) {
        return {
            id: "a67a42c0ff0435776c2873d06ff6ec7cd8940be3",
            mode: "040000",
            name: path_1.default.basename(filePath),
            path: filePath,
            type: "tree",
        };
    }
    else {
        const content = await fs_extra_1.default.readFile(absFilePath);
        const hash = hasha_1.default(content);
        return {
            id: hash,
            mode: "100644",
            name: path_1.default.basename(filePath),
            path: filePath,
            type: "blob",
        };
    }
}
async function doCommitAction(action, options) {
    const absFilePath = path_1.default.join(options.cwd, action.file_path);
    if (action.action === "create" || action.action === "update") {
        await fs_extra_1.default.writeFile(absFilePath, action.content, action.encoding === "base64" ? "base64" : "utf8");
    }
    else if (action.action === "delete") {
        await fs_extra_1.default.unlink(absFilePath);
    }
}
const tree = (options) => express_async_handler_1.default(async (req, res, next) => {
    const basePath = req.query.path;
    const absBasePath = path_1.default.join(options.cwd, basePath);
    if (!(await fs_extra_1.default.pathExists(absBasePath)) &&
        !(await fs_extra_1.default.stat(absBasePath)).isDirectory) {
        return next();
    }
    const files = await fs_extra_1.default.readdir(absBasePath);
    const treeFiles = await Promise.all(files.map(fileName => fileToTree(path_1.default.join(basePath, fileName), options)));
    res.send(treeFiles);
});
const readFile = (options) => express_async_handler_1.default(async (req, res, next) => {
    const absFilePath = path_1.default.join(options.cwd, req.params["0"]);
    if (!(await fs_extra_1.default.pathExists(absFilePath))) {
        return;
    }
    const stat = await fs_extra_1.default.stat(absFilePath);
    if (stat.isDirectory()) {
        return next();
    }
    const content = await fs_extra_1.default.readFile(absFilePath);
    const hash = hasha_1.default(content);
    res.send({
        blob_id: hash,
        commit_id: "",
        content: content.toString("base64"),
        content_sha256: hash,
        encoding: "base64",
        file_name: path_1.default.basename(req.params["0"]),
        file_path: req.params["0"],
        last_commit_id: "",
        ref: "master",
        size: content.length,
    });
});
const commit = (options) => express_async_handler_1.default(async (req, res, next) => {
    const actions = req.body.actions || [];
    await Promise.all(actions.map(action => doCommitAction(action, options)));
    res.send({ ok: true });
});
